@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audiofx_amplitude.cpp: Amplitude effects and dynamic processors.
-// Copyright (C) 1999-2000 Kai Vehmanen (kaiv@wakkanet.fi)
+// Copyright (C) 1999-2000,2003 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,54 +19,69 @@
 
 #include <cmath>
 
-#include <kvutils/message_item.h>
+#include <kvu_message_item.h>
 
 #include "samplebuffer_iterators.h"
 #include "audiofx_amplitude.h"
 
-#include "eca-debug.h"
+#include "eca-logger.h"
 #include "eca-error.h"
 
-EFFECT_AMPLIFY::EFFECT_AMPLIFY (parameter_type multiplier_percent) {
+EFFECT_AMPLITUDE::~EFFECT_AMPLITUDE(void)
+{
+}
+
+EFFECT_AMPLIFY::EFFECT_AMPLIFY (parameter_t multiplier_percent)
+{
   set_parameter(1, multiplier_percent);
 }
 
-void EFFECT_AMPLIFY::set_parameter(int param, parameter_type value) {
+EFFECT_AMPLIFY::~EFFECT_AMPLIFY(void)
+{
+}
+
+void EFFECT_AMPLIFY::set_parameter(int param, parameter_t value) {
   switch (param) {
   case 1: 
-    kerroin = value / 100.0;
+    gain = value / 100.0;
     break;
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_AMPLIFY::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_AMPLIFY::get_parameter(int param) const { 
   switch (param) {
   case 1: 
-    return(kerroin * 100.0);
+    return(gain * 100.0);
   }
   return(0.0);
 }
 
+void EFFECT_AMPLIFY::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  OPERATOR::parameter_description(param, pd);
+}
+
 void EFFECT_AMPLIFY::init(SAMPLE_BUFFER* sbuf) { i.init(sbuf); }
 
 void EFFECT_AMPLIFY::process(void) {
   i.begin();
   while(!i.end()) {
-    *i.current() = *i.current() *  kerroin;
+    *i.current() = *i.current() *  gain;
     i.next();
   }
 }
 
-EFFECT_AMPLIFY_CLIPCOUNT::EFFECT_AMPLIFY_CLIPCOUNT (parameter_type multiplier_percent, int max_clipped) {
+EFFECT_AMPLIFY_CLIPCOUNT::EFFECT_AMPLIFY_CLIPCOUNT (parameter_t multiplier_percent, int max_clipped) {
   set_parameter(1, multiplier_percent);
   set_parameter(2, max_clipped);
   num_of_clipped = 0;
 }
 
-void EFFECT_AMPLIFY_CLIPCOUNT::set_parameter(int param, parameter_type value) {
+void EFFECT_AMPLIFY_CLIPCOUNT::set_parameter(int param, parameter_t value)
+{
   switch (param) {
   case 1: 
-    kerroin = value / 100.0;
+    gain = value / 100.0;
     break;
   case 2:
     maxnum_of_clipped = (int)value;
@@ -74,10 +89,11 @@ void EFFECT_AMPLIFY_CLIPCOUNT::set_parameter(int param, parameter_type value) {
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_AMPLIFY_CLIPCOUNT::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_AMPLIFY_CLIPCOUNT::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(kerroin * 100.0);
+    return(gain * 100.0);
   case 2:
     return(maxnum_of_clipped);
   }
@@ -86,10 +102,11 @@ CHAIN_OPERATOR::parameter_type EFFECT_AMPLIFY_CLIPCOUNT::get_parameter(int param
 
 void EFFECT_AMPLIFY_CLIPCOUNT::init(SAMPLE_BUFFER* sbuf) { i.init(sbuf); }
 
-void EFFECT_AMPLIFY_CLIPCOUNT::process(void) {
+void EFFECT_AMPLIFY_CLIPCOUNT::process(void)
+{
   i.begin();
   while(!i.end()) {
-    *i.current() = *i.current() *  kerroin;
+    *i.current() = *i.current() *  gain;
     if (*i.current() > SAMPLE_SPECS::impl_max_value ||
 	*i.current() < SAMPLE_SPECS::impl_min_value) {
       num_of_clipped++;
@@ -106,55 +123,92 @@ void EFFECT_AMPLIFY_CLIPCOUNT::process(void) {
     otemp << "(audiofx_amplitude) WARNING! Signal is clipping! ";
     otemp << num_of_clipped;
     otemp << " consecutive clipped samples.";
-    ecadebug->msg(otemp.to_string());
+    ECA_LOG_MSG(ECA_LOGGER::info, otemp.to_string());
   }
 }
 
-EFFECT_AMPLIFY_CHANNEL::EFFECT_AMPLIFY_CHANNEL (parameter_type multiplier_percent, int channel) {
+void EFFECT_AMPLIFY_CLIPCOUNT::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  OPERATOR::parameter_description(param, pd);
+}
+
+EFFECT_AMPLIFY_CHANNEL::EFFECT_AMPLIFY_CHANNEL (parameter_t multiplier_percent, int channel)
+{
   set_parameter(1, multiplier_percent);
   set_parameter(2, channel);
 }
 
-void EFFECT_AMPLIFY_CHANNEL::set_parameter(int param, parameter_type value) {
+void EFFECT_AMPLIFY_CHANNEL::set_parameter(int param, parameter_t value)
+{
   switch (param) {
-  case 1: 
-    kerroin = value / 100.0;
-    break;
-
- case 2: 
-    channel_rep = static_cast<int>(value);
-    assert(channel_rep > 0);
-    channel_rep--;
-    break;
+    case 1: 
+      gain = value / 100.0;
+      break;
+      
+    case 2: 
+      if (value > 0) {
+	channel_rep = static_cast<int>(value);
+	channel_rep--;
+	break;
+      }
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_AMPLIFY_CHANNEL::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_AMPLIFY_CHANNEL::get_parameter(int param) const
+{ 
   switch (param) {
   case 1: 
-    return(kerroin * 100.0);
+    return(gain * 100.0);
 
   case 2: 
-    return(static_cast<parameter_type>(channel_rep + 1));
+    return(static_cast<parameter_t>(channel_rep + 1));
   }
   return(0.0);
 }
 
-void EFFECT_AMPLIFY_CHANNEL::init(SAMPLE_BUFFER *insample) { i.init(insample); }
+void EFFECT_AMPLIFY_CHANNEL::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  OPERATOR::parameter_description(param, pd);
 
-void EFFECT_AMPLIFY_CHANNEL::process(void) {
-  i.begin(channel_rep);
-  while(!i.end()) {
-    *i.current() = *i.current() * kerroin;
-    i.next();
+  switch (param) {
+  case 2: 
+    {
+      pd->default_value = 1.0f;
+      pd->bounded_above = false;
+      pd->bounded_below = true;
+      pd->lower_bound = 1.0f;
+      pd->integer = true;
+    }
   }
 }
 
-EFFECT_LIMITER::EFFECT_LIMITER (parameter_type limiting_percent) {
+void EFFECT_AMPLIFY_CHANNEL::init(SAMPLE_BUFFER *insample)
+{
+  EFFECT_BASE::init(insample);
+  i.init(insample);
+}
+
+void EFFECT_AMPLIFY_CHANNEL::process(void)
+{
+  if (channel_rep >= 0 && channel_rep < channels()) {
+    i.begin(channel_rep);
+    while(!i.end()) {
+      *i.current() = *i.current() * gain;
+      i.next();
+    }
+  }
+}
+
+EFFECT_LIMITER::EFFECT_LIMITER (parameter_t limiting_percent)
+{
   set_parameter(1, limiting_percent);
 }
 
-void EFFECT_LIMITER::set_parameter(int param, parameter_type value) {
+EFFECT_LIMITER::~EFFECT_LIMITER(void)
+{
+}
+
+void EFFECT_LIMITER::set_parameter(int param, parameter_t value) {
   switch (param) {
   case 1:
     limit_rep = value / 100.0;
@@ -162,7 +216,7 @@ void EFFECT_LIMITER::set_parameter(int param, parameter_type value) {
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_LIMITER::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_LIMITER::get_parameter(int param) const { 
   switch (param) {
   case 1: 
     return(limit_rep * 100.0);
@@ -170,6 +224,11 @@ CHAIN_OPERATOR::parameter_type EFFECT_LIMITER::get_parameter(int param) const {
   return(0.0);
 }
 
+void EFFECT_LIMITER::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  OPERATOR::parameter_description(param, pd);
+}
+
 void EFFECT_LIMITER::init(SAMPLE_BUFFER* sbuf) { i.init(sbuf); }
 
 void EFFECT_LIMITER::process(void) {
@@ -188,7 +247,7 @@ void EFFECT_LIMITER::process(void) {
   }
 }
 
-EFFECT_COMPRESS::EFFECT_COMPRESS (parameter_type compress_rate, parameter_type thold) {
+EFFECT_COMPRESS::EFFECT_COMPRESS (parameter_t compress_rate, parameter_t thold) {
   set_parameter(1, compress_rate);
   set_parameter(2, thold);
 
@@ -206,7 +265,7 @@ EFFECT_COMPRESS::EFFECT_COMPRESS (const EFFECT_COMPRESS& x) {
   lastout = x.lastout;
 }
 
-void EFFECT_COMPRESS::set_parameter(int param, parameter_type value) {
+void EFFECT_COMPRESS::set_parameter(int param, parameter_t value) {
   switch (param) {
   case 1: 
     crate = pow(2.0, value / 6.0);
@@ -217,7 +276,7 @@ void EFFECT_COMPRESS::set_parameter(int param, parameter_type value) {
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_COMPRESS::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_COMPRESS::get_parameter(int param) const { 
   switch (param) {
   case 1: 
     return((log (crate)) / (log (2.0f)) * 6.0);
@@ -227,17 +286,50 @@ CHAIN_OPERATOR::parameter_type EFFECT_COMPRESS::get_parameter(int param) const {
   return(0.0);
 }
 
-void EFFECT_COMPRESS::init(SAMPLE_BUFFER *insample) { 
+void EFFECT_COMPRESS::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  switch (param) {
+  case 1:
+    pd->default_value = 1.0f;
+    pd->description = "compression-rate-dB";
+    pd->bounded_above = true;
+    pd->upper_bound = 99.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = true;
+    pd->output = false;
+    break;
+  case 2:
+    pd->default_value = 30.0f;
+    pd->description = "threshold-%";
+    pd->bounded_above = true;
+    pd->upper_bound = 100.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  default: {}
+  }
+}
+
+void EFFECT_COMPRESS::init(SAMPLE_BUFFER *insample)
+{
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
+  set_samples_per_second(samples_per_second());
 
   lastin.resize(insample->number_of_channels());
   lastout.resize(insample->number_of_channels());
 } 
 
-void EFFECT_COMPRESS::process(void) {
+void EFFECT_COMPRESS::process(void)
+{
   i.begin();
   while(!i.end()) {
     if (first_time) {
@@ -266,7 +358,12 @@ void EFFECT_COMPRESS::process(void) {
   }
 }
 
-EFFECT_NOISEGATE::EFFECT_NOISEGATE (parameter_type thlevel_percent, parameter_type thtime, parameter_type a, parameter_type h, parameter_type r) {
+EFFECT_NOISEGATE::EFFECT_NOISEGATE (parameter_t thlevel_percent, 
+				    parameter_t thtime, 
+				    parameter_t a, 
+				    parameter_t h, 
+				    parameter_t r)
+{
   // map_parameters();
 
   set_parameter(1, thlevel_percent);
@@ -276,58 +373,66 @@ EFFECT_NOISEGATE::EFFECT_NOISEGATE (parameter_type thlevel_percent, parameter_ty
   set_parameter(5, r);
 }
 
-void EFFECT_NOISEGATE::set_parameter(int param, parameter_type value) {
+void EFFECT_NOISEGATE::set_parameter(int param, parameter_t value) {
   switch (param) {
   case 1: 
     th_level = SAMPLE_SPECS::max_amplitude * (value / 100.0);
     break;
   case 2: 
-    th_time = (value * (parameter_type)samples_per_second() / 1000.0);
+    th_time = (value * (parameter_t)samples_per_second() / 1000.0);
     break;
   case 3: 
-    atime = (value * (parameter_type)samples_per_second() / 1000.0);
+    atime = (value * (parameter_t)samples_per_second() / 1000.0);
     break;
   case 4: 
-    htime = (value * (parameter_type)samples_per_second() / 1000.0);
+    htime = (value * (parameter_t)samples_per_second() / 1000.0);
     break;
   case 5: 
-    rtime = (value * (parameter_type)samples_per_second() / 1000.0);
+    rtime = (value * (parameter_t)samples_per_second() / 1000.0);
     break;
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_NOISEGATE::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_NOISEGATE::get_parameter(int param) const
+{ 
   switch (param) {
   case 1: 
-    return(th_level * 100.0 / (parameter_type)SAMPLE_SPECS::max_amplitude);
+    return(th_level * 100.0 / (parameter_t)SAMPLE_SPECS::max_amplitude);
   case 2: 
-    return(th_time * 1000.0 / (parameter_type)samples_per_second());
+    return(th_time * 1000.0 / (parameter_t)samples_per_second());
   case 3: 
-    return(atime * 1000.0 / (parameter_type)samples_per_second());
+    return(atime * 1000.0 / (parameter_t)samples_per_second());
   case 4: 
-    return(htime * 1000.0 / (parameter_type)samples_per_second());
+    return(htime * 1000.0 / (parameter_t)samples_per_second());
   case 5: 
-    return(rtime * 1000.0 / (parameter_type)samples_per_second());
+    return(rtime * 1000.0 / (parameter_t)samples_per_second());
   }
   return(0.0);
 }
 
-void EFFECT_NOISEGATE::init(SAMPLE_BUFFER *insample) {
+void EFFECT_NOISEGATE::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  OPERATOR::parameter_description(param, pd);
+}
+
+void EFFECT_NOISEGATE::init(SAMPLE_BUFFER *insample)
+{
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
+  set_samples_per_second(samples_per_second());
 
   th_time_lask.resize(insample->number_of_channels());
   attack_lask.resize(insample->number_of_channels());
   hold_lask.resize(insample->number_of_channels());
   release_lask.resize(insample->number_of_channels());
-  kerroin.resize(insample->number_of_channels());
+  gain.resize(insample->number_of_channels());
 
   ng_status.resize(insample->number_of_channels(), int(ng_waiting));
 }
 
-void EFFECT_NOISEGATE::process(void) {
+void EFFECT_NOISEGATE::process(void)
+{
   i.begin();
   while(!i.end()) {
     bool below = fabs(*i.current()) <= th_level;
@@ -344,7 +449,7 @@ void EFFECT_NOISEGATE::process(void) {
 	    if (th_time_lask[i.channel()] >= th_time) {
 	      th_time_lask[i.channel()] = 0.0;
 	      ng_status[i.channel()] = ng_attacking;
-	      ecadebug->msg(ECA_DEBUG::user_objects,"(audiofx) noisegate - from waiting to attacking");
+	      ECA_LOG_MSG(ECA_LOGGER::user_objects,"(audiofx) noisegate - from waiting to attacking");
 	    }
 	  }
 	  else {
@@ -360,19 +465,19 @@ void EFFECT_NOISEGATE::process(void) {
 	{
 	  if (below) {
 	    attack_lask[i.channel()]++;
-	    kerroin[i.channel()] = (1.0 - (attack_lask[i.channel()] / atime));
+	    gain[i.channel()] = (1.0 - (attack_lask[i.channel()] / atime));
 	    if (attack_lask[i.channel()] >= atime) {
 	      attack_lask[i.channel()] = 0.0;
 	      ng_status[i.channel()] = ng_active;
-	      kerroin[i.channel()] = 0.0;
-	      ecadebug->msg(ECA_DEBUG::user_objects,"(audiofx) noisegate - from attack to active");
+	      gain[i.channel()] = 0.0;
+	      ECA_LOG_MSG(ECA_LOGGER::user_objects,"(audiofx) noisegate - from attack to active");
 	    }
-	    *i.current() = *i.current() * kerroin[i.channel()];
+	    *i.current() = *i.current() * gain[i.channel()];
 	  }
 	  else {
 	    attack_lask[i.channel()] = 0;
 	    ng_status[i.channel()] = ng_waiting;
-	    ecadebug->msg(ECA_DEBUG::user_objects,"(audiofx) noisegate - from attack to waiting");
+	    ECA_LOG_MSG(ECA_LOGGER::user_objects,"(audiofx) noisegate - from attack to waiting");
 	  }
 	  break;
 	}
@@ -384,7 +489,7 @@ void EFFECT_NOISEGATE::process(void) {
 	{
 	  if (below == false) {
 	    ng_status[i.channel()] = ng_holding;
-	    ecadebug->msg(ECA_DEBUG::user_objects,"(audiofx) noisegate - from active to holding");
+	    ECA_LOG_MSG(ECA_LOGGER::user_objects,"(audiofx) noisegate - from active to holding");
 	  }
 	  *i.current() = *i.current() * 0.0;
 	  break;
@@ -400,7 +505,7 @@ void EFFECT_NOISEGATE::process(void) {
 	    if (hold_lask[i.channel()] >= htime) {
 	      hold_lask[i.channel()] = 0.0;
 	      ng_status[i.channel()] = ng_releasing;
-	      ecadebug->msg(ECA_DEBUG::user_objects,"(audiofx) noisegate - from holding to release");
+	      ECA_LOG_MSG(ECA_LOGGER::user_objects,"(audiofx) noisegate - from holding to release");
 	    }
 	  }
 	  *i.current() = *i.current() * 0.0;
@@ -413,13 +518,13 @@ void EFFECT_NOISEGATE::process(void) {
 	// ---
 	{
 	  release_lask[i.channel()]++;
-	  kerroin[i.channel()] = release_lask[i.channel()] / rtime;
+	  gain[i.channel()] = release_lask[i.channel()] / rtime;
 	  if (release_lask[i.channel()] >= rtime) {
 	    release_lask[i.channel()] = 0.0;
 	    ng_status[i.channel()] = ng_waiting;
-	    ecadebug->msg(ECA_DEBUG::user_objects,"(audiofx) noisegate - from releasing to waiting");
+	    ECA_LOG_MSG(ECA_LOGGER::user_objects,"(audiofx) noisegate - from releasing to waiting");
 	  }
-	  *i.current() = *i.current() * kerroin[i.channel()];
+	  *i.current() = *i.current() * gain[i.channel()];
 	  break;
 	}
       }
@@ -428,30 +533,33 @@ void EFFECT_NOISEGATE::process(void) {
   }
 }
 
-EFFECT_NORMAL_PAN::EFFECT_NORMAL_PAN (parameter_type right_percent) {
+EFFECT_NORMAL_PAN::EFFECT_NORMAL_PAN (parameter_t right_percent)
+{
   set_parameter(1, right_percent);
 }
 
-void EFFECT_NORMAL_PAN::set_parameter(int param, parameter_type value) {
+void EFFECT_NORMAL_PAN::set_parameter(int param, parameter_t value)
+{
   switch (param) {
   case 1: 
     right_percent_rep = value;
     if (value == 50.0) {
-      l_kerroin = r_kerroin = 1.0;
+      l_gain = r_gain = 1.0;
     }
     else if (value < 50.0) {
-      l_kerroin = 1.0;
-      r_kerroin = value / 50.0;
+      l_gain = 1.0;
+      r_gain = value / 50.0;
     }
     else if (value > 50.0) {
-      r_kerroin = 1.0;
-      l_kerroin = (100.0 - value) / 50.0;
+      r_gain = 1.0;
+      l_gain = (100.0 - value) / 50.0;
     }
     break;
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_NORMAL_PAN::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_NORMAL_PAN::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
     return(right_percent_rep);
@@ -459,18 +567,35 @@ CHAIN_OPERATOR::parameter_type EFFECT_NORMAL_PAN::get_parameter(int param) const
   return(0.0);
 }
 
+void EFFECT_NORMAL_PAN::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  OPERATOR::parameter_description(param, pd);
+
+  switch (param) {
+  case 1: 
+    {
+      pd->default_value = 50.0f;
+      pd->bounded_above = true;
+      pd->upper_bound = 100.0f;
+      pd->bounded_below = true;
+      pd->lower_bound = 0.0f;
+    }
+  }
+}
+
 void EFFECT_NORMAL_PAN::init(SAMPLE_BUFFER *insample) { i.init(insample); }
 
-void EFFECT_NORMAL_PAN::process(void) {
+void EFFECT_NORMAL_PAN::process(void)
+{
   i.begin(0);
   while(!i.end()) {
-    *i.current() = *i.current() * l_kerroin;
+    *i.current() = *i.current() * l_gain;
     i.next();
   }
 
   i.begin(1);
   while(!i.end()) {
-    *i.current() = *i.current() * r_kerroin;
+    *i.current() = *i.current() * r_gain;
     i.next();
   }
 }
