@@ -16,102 +16,144 @@
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+//
+// ------------------------------------------------------------------------
+// 
+// History: 
+//
+// 2003-08-21 Kai Vehmanen
+//     - Fixed a timing resolution bug in the envelope modulation code.
+//       This involved changing from float to fixed point presentation for 
+//       the position counters to avoid loss of precision (led to unexpected
+//       drift in pulse timing).
+// 2000-11-01 Rob Coker
+//     - Initial version.
+//
 // ------------------------------------------------------------------------
 
 #include <cmath>
 
-#include <kvutils/message_item.h>
+#include <kvu_message_item.h>
 
 #include "samplebuffer_iterators.h"
 #include "audiofx_envelope_modulation.h"
 
-#include "eca-debug.h"
+#include "eca-logger.h"
 #include "eca-error.h"
 
-EFFECT_PULSE_GATE::EFFECT_PULSE_GATE (parameter_type freq_Hz,
-				      parameter_type onTime_percent) {
-  set_parameter(1, freq_Hz);
-  set_parameter(2, onTime_percent);
-  currentTime = 0.0;
+EFFECT_ENV_MOD::~EFFECT_ENV_MOD(void)
+{
 }
 
-void EFFECT_PULSE_GATE::set_parameter(int param, parameter_type value) {
+EFFECT_PULSE_GATE::EFFECT_PULSE_GATE (parameter_t freq_Hz,
+				      parameter_t onTime_percent)
+{
+  set_parameter(1, freq_Hz);
+  set_parameter(2, onTime_percent);
+
+  freq_rep = 1.0;
+  on_time_rep = 0;
+  current_rep = 0;
+  period_rep = 0;
+  on_from_rep = 0;
+}
+
+EFFECT_PULSE_GATE::~EFFECT_PULSE_GATE(void)
+{
+}
+
+void EFFECT_PULSE_GATE::set_samples_per_second(SAMPLE_SPECS::sample_rate_t v)
+{
+  /* NOP, see audiofx.cpp:set_samples_per_second(); */
+  EFFECT_ENV_MOD::set_samples_per_second(v);
+}
+
+void EFFECT_PULSE_GATE::set_parameter(int param, parameter_t value)
+{
   switch (param) {
   case 1: 
     if (value > 0)
       {
-	period = 1.0/value; // seconds
+	freq_rep = value;
+	period_rep = static_cast<long int>(1.0f / freq_rep * samples_per_second() + 0.5f); // samples
       }
     else
       {
 	MESSAGE_ITEM otemp;
 	otemp << "(audiofx_envelope_modulation) WARNING! Frequency must be greater than 0! ";
-	ecadebug->msg(otemp.to_string());
+	ECA_LOG_MSG(ECA_LOGGER::user_objects, otemp.to_string());
       }
     break;
+
   case 2:
     if ((value > 0) && (value < 100))
       {
-	stopTime = (value/100.0)*period; // seconds
+	on_time_rep = value;
+	on_from_rep = static_cast<long int>((on_time_rep / 100.0) * period_rep + 0.5f);
       }
     else
       {
 	MESSAGE_ITEM otemp;
 	otemp << "(audiofx_envelope_modulation) WARNING! on time must be between 0 and 100 inclusive! ";
-	ecadebug->msg(otemp.to_string());
+	ECA_LOG_MSG(ECA_LOGGER::user_objects, otemp.to_string());
       }
     break;
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_PULSE_GATE::get_parameter(int param) const {
+CHAIN_OPERATOR::parameter_t EFFECT_PULSE_GATE::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(1.0/period);
-    break;
+    return(freq_rep);
+
   case 2:
-    return (stopTime/period)*100.0;
-    break;
+    return (on_time_rep);
   }
   return(0.0);
 }
 
-void EFFECT_PULSE_GATE::init(SAMPLE_BUFFER* sbuf) { 
+void EFFECT_PULSE_GATE::init(SAMPLE_BUFFER* sbuf)
+{ 
   i.init(sbuf); 
-  set_samples_per_second(sbuf->sample_rate());
   set_channels(sbuf->number_of_channels());
-  incrTime = 1.0/samples_per_second();
+  EFFECT_ENV_MOD::init(sbuf);
 }
 
-void EFFECT_PULSE_GATE::process(void) {
+void EFFECT_PULSE_GATE::process(void)
+{
   i.begin(); // iterate through all samples, one sample-frame at a
              // time (interleaved)
   while(!i.end()) {
-    currentTime += incrTime;
-    if (currentTime > period)
-      {
-	currentTime = 0.0;
-      }
-    if (currentTime > stopTime)
-      {
-	for(int n = 0; n < channels(); n++) {
-	  *i.current(n) = 0.0;
-	}
+    ++current_rep;
+    if (current_rep >= period_rep) {
+	current_rep = 0;
+    }
+    if (current_rep > on_from_rep) {
+      for(int n = 0; n < channels(); n++) {
+	*i.current(n) = 0.0;
       }
+    }
     i.next();
   }
 }
 
-EFFECT_PULSE_GATE_BPM::EFFECT_PULSE_GATE_BPM (parameter_type bpm,
-					      parameter_type ontime_percent) {
+EFFECT_PULSE_GATE_BPM::EFFECT_PULSE_GATE_BPM (parameter_t bpm,
+					      parameter_t ontime_percent)
+{
   set_parameter(1, bpm);
   set_parameter(2, ontime_percent);
 }
 
-void EFFECT_PULSE_GATE_BPM::set_parameter(int param, parameter_type value) {
+EFFECT_PULSE_GATE_BPM::~EFFECT_PULSE_GATE_BPM(void)
+{
+}
+
+void EFFECT_PULSE_GATE_BPM::set_parameter(int param, parameter_t value)
+{
   switch (param) {
   case 1: 
-    pulsegate_rep.set_parameter(1, value / 60.0);
+    pulsegate_rep.set_parameter(1, value / 60.0f);
     break;
   case 2:
     pulsegate_rep.set_parameter(2, value);
@@ -119,10 +161,10 @@ void EFFECT_PULSE_GATE_BPM::set_parameter(int param, parameter_type value) {
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_PULSE_GATE_BPM::get_parameter(int param) const {
+CHAIN_OPERATOR::parameter_t EFFECT_PULSE_GATE_BPM::get_parameter(int param) const {
   switch (param) {
   case 1: 
-    return (pulsegate_rep.get_parameter(1) * 60.0);
+    return (pulsegate_rep.get_parameter(1) * 60.0f);
     break;
   case 2:
     return (pulsegate_rep.get_parameter(2));
@@ -131,17 +173,34 @@ CHAIN_OPERATOR::parameter_type EFFECT_PULSE_GATE_BPM::get_parameter(int param) c
   return(0.0);
 }
 
-void EFFECT_PULSE_GATE_BPM::init(SAMPLE_BUFFER* sbuf) { pulsegate_rep.init(sbuf); }
+void EFFECT_PULSE_GATE_BPM::init(SAMPLE_BUFFER* sbuf)
+{ 
+  pulsegate_rep.init(sbuf); 
+  EFFECT_ENV_MOD::init(sbuf);
+}
+
 void EFFECT_PULSE_GATE_BPM::process(void) { pulsegate_rep.process(); }
 
-EFFECT_TREMOLO::EFFECT_TREMOLO (parameter_type freq_bpm,
-				      parameter_type depth_percent) {
+void EFFECT_PULSE_GATE_BPM::set_samples_per_second(SAMPLE_SPECS::sample_rate_t v)
+{
+  pulsegate_rep.set_samples_per_second(v);
+  EFFECT_ENV_MOD::set_samples_per_second(v);
+}
+
+EFFECT_TREMOLO::EFFECT_TREMOLO (parameter_t freq_bpm,
+				parameter_t depth_percent)
+{
   set_parameter(1, freq_bpm);
   set_parameter(2, depth_percent);
   currentTime = 0.0;
 }
 
-void EFFECT_TREMOLO::set_parameter(int param, parameter_type value) {
+EFFECT_TREMOLO::~EFFECT_TREMOLO(void)
+{
+}
+
+void EFFECT_TREMOLO::set_parameter(int param, parameter_t value)
+{
   switch (param) {
   case 1:
     if (value > 0)
@@ -152,7 +211,7 @@ void EFFECT_TREMOLO::set_parameter(int param, parameter_type value) {
       {
 	MESSAGE_ITEM otemp;
 	otemp << "(audiofx_envelope_modulation) WARNING! bpm must be greater than 0! ";
-	ecadebug->msg(otemp.to_string());
+	ECA_LOG_MSG(ECA_LOGGER::info, otemp.to_string());
       }
     break;
   case 2:
@@ -161,7 +220,8 @@ void EFFECT_TREMOLO::set_parameter(int param, parameter_type value) {
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_TREMOLO::get_parameter(int param) const {
+CHAIN_OPERATOR::parameter_t EFFECT_TREMOLO::get_parameter(int param) const
+{
   switch (param) {
   case 1:
     return freq*120;
@@ -173,14 +233,16 @@ CHAIN_OPERATOR::parameter_type EFFECT_TREMOLO::get_parameter(int param) const {
   return(0.0);
 }
 
-void EFFECT_TREMOLO::init(SAMPLE_BUFFER* sbuf) {
+void EFFECT_TREMOLO::init(SAMPLE_BUFFER* sbuf)
+{
   i.init(sbuf);
-  set_samples_per_second(sbuf->sample_rate());
   set_channels(sbuf->number_of_channels());
   incrTime = 1.0/samples_per_second();
+  EFFECT_ENV_MOD::init(sbuf);
 }
 
-void EFFECT_TREMOLO::process(void) {
+void EFFECT_TREMOLO::process(void)
+{
   i.begin(); // iterate through all samples, one sample-frame at a
              // time (interleaved)
   while(!i.end())
