@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audiofx_timebased.cpp: Routines for time-based effects.
-// Copyright (C) 1999-2000 Kai Vehmanen (kaiv@wakkanet.fi)
+// Copyright (C) 1999-2003 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,17 +17,20 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 // ------------------------------------------------------------------------
 
+#include <assert.h>
 #include <string>
 
-#include <kvutils.h>
+#include <kvu_dbc.h>
+#include <kvu_utils.h>
 
-#include "eca-debug.h"
+#include "eca-logger.h"
 
 #include "samplebuffer_iterators.h"
 #include "audiofx_timebased.h"
 
-EFFECT_DELAY::EFFECT_DELAY (CHAIN_OPERATOR::parameter_type delay_time, int surround_mode, 
-			    int num_of_delays, CHAIN_OPERATOR::parameter_type mix_percent) 
+EFFECT_DELAY::EFFECT_DELAY (CHAIN_OPERATOR::parameter_t delay_time, int surround_mode, 
+			    int num_of_delays, CHAIN_OPERATOR::parameter_t mix_percent,
+			    CHAIN_OPERATOR::parameter_t feedback_percent) 
 {
   laskuri = 0.0;
 
@@ -35,9 +38,77 @@ EFFECT_DELAY::EFFECT_DELAY (CHAIN_OPERATOR::parameter_type delay_time, int surro
   set_parameter(2, surround_mode);
   set_parameter(3, num_of_delays);
   set_parameter(4, mix_percent);
+  set_parameter(5, feedback_percent);
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_DELAY::get_parameter(int param) const { 
+void EFFECT_DELAY::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  switch (param) {
+  case 1:
+    pd->default_value = 100.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = false;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 2:
+    pd->default_value = 0.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = true;
+    pd->upper_bound = 1.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = true;
+    pd->integer = true;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 3:
+    pd->default_value = 1.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = false;
+    // pd->upper_bound = 0.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 1.0f;
+    pd->toggled = false;
+    pd->integer = true;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 4:
+    pd->default_value = 50.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = true;
+    pd->upper_bound = 100.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 5:
+    pd->default_value = 100.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = true;
+    pd->upper_bound = 100.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  default: {}
+  }
+}
+
+CHAIN_OPERATOR::parameter_t EFFECT_DELAY::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
     return(dtime_msec);
@@ -47,19 +118,22 @@ CHAIN_OPERATOR::parameter_type EFFECT_DELAY::get_parameter(int param) const {
     return(dnum);
   case 4:
     return(mix * 100.0);
+  case 5:
+    return(feedback * 100.0);
   }
   return(0.0);
 }
 
-void EFFECT_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1:
     {
       dtime_msec = value;
-      dtime = dtime_msec * (CHAIN_OPERATOR::parameter_type)samples_per_second() / 1000;
-      vector<vector<SINGLE_BUFFER> >::iterator p = buffer.begin();
+      dtime = dtime_msec * (CHAIN_OPERATOR::parameter_t)samples_per_second() / 1000;
+      std::vector<std::vector<SINGLE_BUFFER> >::iterator p = buffer.begin();
       while(p != buffer.end()) {
-	vector<SINGLE_BUFFER>::iterator q = p->begin();
+	std::vector<SINGLE_BUFFER>::iterator q = p->begin();
 	while(q != p->end()) {
 	  if (q->size() > dtime) {
 	    q->resize(static_cast<unsigned int>(dtime));
@@ -80,7 +154,7 @@ void EFFECT_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_type value
     {
       if (value != 0.0) dnum = static_cast<long int>(value);
       else dnum = 1.0;
-      vector<vector<SINGLE_BUFFER> >::iterator p = buffer.begin();
+      std::vector<std::vector<SINGLE_BUFFER> >::iterator p = buffer.begin();
       while(p != buffer.end()) {
 	p->resize(static_cast<unsigned int>(dnum));
 	++p;
@@ -92,31 +166,47 @@ void EFFECT_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_type value
   case 4:
     mix = value / 100.0;
     break;
+
+  case 5:
+    if (value == 0 || value > 100) {
+      feedback = 1.0;
+    } else {
+    feedback = value / 100.0;
+    }
+    break;
   }
 }
 
-void EFFECT_DELAY::init(SAMPLE_BUFFER* insample) {
+void EFFECT_DELAY::init(SAMPLE_BUFFER* insample)
+{
   l.init(insample);
   r.init(insample);
 
-  set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
+  EFFECT_BASE::init(insample);
+
   set_parameter(1, dtime_msec);
 
-  buffer.resize(2, vector<SINGLE_BUFFER> (static_cast<unsigned int>(dnum)));
+  buffer.resize(2, std::vector<SINGLE_BUFFER> (static_cast<unsigned int>(dnum)));
 }
 
-void EFFECT_DELAY::process(void) {
+void EFFECT_DELAY::process(void)
+{
   l.begin(SAMPLE_SPECS::ch_left);
   r.begin(SAMPLE_SPECS::ch_right);
 
   while(!l.end() && !r.end()) {
-    SAMPLE_SPECS::sample_type temp2_left = 0.0;
-    SAMPLE_SPECS::sample_type temp2_right = 0.0;
+    SAMPLE_SPECS::sample_t temp2_left = 0.0;
+    SAMPLE_SPECS::sample_t temp2_right = 0.0;
+
+    // Initializing the feedback factor to one. (x*1 = x)
+    SAMPLE_SPECS::sample_t feedfact = 1;
 
     for(int nm2 = 0; nm2 < dnum; nm2++) {
-      SAMPLE_SPECS::sample_type temp_left = 0.0;
-      SAMPLE_SPECS::sample_type temp_right = 0.0;
+      SAMPLE_SPECS::sample_t temp_left = 0.0;
+      SAMPLE_SPECS::sample_t temp_right = 0.0;
+
+      // Preparing the factor...
+      feedfact *= feedback;
 
       if (laskuri >= dtime * (nm2 + 1)) {
  
@@ -155,6 +245,11 @@ void EFFECT_DELAY::process(void) {
 	    break;
 	}
 	} // switch
+
+	// Applying the reduction.
+	temp_left *= feedfact;
+	temp_right *= feedfact;
+
 	buffer[SAMPLE_SPECS::ch_left][nm2].pop_front();
 	buffer[SAMPLE_SPECS::ch_right][nm2].pop_front();
       }
@@ -175,19 +270,63 @@ void EFFECT_DELAY::process(void) {
   }
 }
 
-EFFECT_MULTITAP_DELAY::EFFECT_MULTITAP_DELAY (CHAIN_OPERATOR::parameter_type delay_time, 
+EFFECT_MULTITAP_DELAY::EFFECT_MULTITAP_DELAY (CHAIN_OPERATOR::parameter_t delay_time, 
 					      int num_of_delays, 
-					      CHAIN_OPERATOR::parameter_type mix_percent)
+					      CHAIN_OPERATOR::parameter_t mix_percent)
   : 
   delay_index(0),
   filled (0),
-  buffer (0) {
+  buffer (0)
+{
   set_parameter(1, delay_time);
   set_parameter(2, num_of_delays);
   set_parameter(3, mix_percent);
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_MULTITAP_DELAY::get_parameter(int param) const { 
+void EFFECT_MULTITAP_DELAY::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  switch (param) {
+  case 1:
+    pd->default_value = 100.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = false;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 2:
+    pd->default_value = 1.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = false;
+    // pd->upper_bound = 0.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 1.0f;
+    pd->toggled = false;
+    pd->integer = true;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 3:
+    pd->default_value = 50.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = true;
+    pd->upper_bound = 100.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  default: {}
+  }
+}
+
+CHAIN_OPERATOR::parameter_t EFFECT_MULTITAP_DELAY::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
     return(dtime_msec);
@@ -199,13 +338,14 @@ CHAIN_OPERATOR::parameter_type EFFECT_MULTITAP_DELAY::get_parameter(int param) c
   return(0.0);
 }
 
-void EFFECT_MULTITAP_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_MULTITAP_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1:
     {
       dtime_msec = value;
-      dtime = static_cast<long int>(dtime_msec * (CHAIN_OPERATOR::parameter_type)samples_per_second() / 1000);
-      assert(buffer.size() == filled.size());
+      dtime = static_cast<long int>(dtime_msec * (CHAIN_OPERATOR::parameter_t)samples_per_second() / 1000);
+      DBC_CHECK(buffer.size() == filled.size());
       for(int n = 0; n < static_cast<int>(buffer.size()); n++) {
 	if ((dtime * dnum) > static_cast<int>(buffer[n].size())) {
 	  buffer[n].resize(dtime * dnum);
@@ -222,7 +362,7 @@ void EFFECT_MULTITAP_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_t
     {
       if (value != 0.0) dnum = static_cast<long int>(value);
       else dnum = 1;
-      assert(buffer.size() == filled.size());
+      DBC_CHECK(buffer.size() == filled.size());
       for(int n = 0; n < static_cast<int>(buffer.size()); n++) {
 	if ((dtime * dnum) > static_cast<int>(buffer[n].size())) {
 	  buffer[n].resize(dtime * dnum);
@@ -244,13 +384,13 @@ void EFFECT_MULTITAP_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_t
 void EFFECT_MULTITAP_DELAY::init(SAMPLE_BUFFER* insample) {
   i.init(insample);
 
-  set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
+  EFFECT_BASE::init(insample);
+
   set_parameter(1, dtime_msec);
 
   delay_index.resize(channels(), dtime * dnum - 1);
-  filled.resize(channels(), vector<bool> (dnum, false));
-  buffer.resize(channels(), vector<SAMPLE_SPECS::sample_type> (dtime *
+  filled.resize(channels(), std::vector<bool> (dnum, false));
+  buffer.resize(channels(), std::vector<SAMPLE_SPECS::sample_t> (dtime *
 							       dnum));
 }
 
@@ -260,11 +400,11 @@ void EFFECT_MULTITAP_DELAY::process(void) {
   i.begin();
   while(!i.end()) {
     for(int n = 0; n < channels(); n++) {
-      SAMPLE_SPECS::sample_type temp1 = 0.0;
+      SAMPLE_SPECS::sample_t temp1 = 0.0;
       for(int nm2 = 0; nm2 < dnum; nm2++) {
 	if (filled[n][nm2] == true) {
-	  assert((delay_index[n] + nm2 * dtime) % len >= 0);
-	  assert((delay_index[n] + nm2 * dtime) % len < len);
+	  DBC_CHECK((delay_index[n] + nm2 * dtime) % len >= 0);
+	  DBC_CHECK((delay_index[n] + nm2 * dtime) % len < len);
 	  temp1 += buffer[n][(delay_index[n] + nm2 * dtime) % len];
 	}
       }
@@ -281,11 +421,31 @@ void EFFECT_MULTITAP_DELAY::process(void) {
   }
 }
 
-EFFECT_FAKE_STEREO::EFFECT_FAKE_STEREO (CHAIN_OPERATOR::parameter_type delay_time) {
+EFFECT_FAKE_STEREO::EFFECT_FAKE_STEREO (CHAIN_OPERATOR::parameter_t delay_time)
+{
    set_parameter(1, delay_time);
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_FAKE_STEREO::get_parameter(int param) const { 
+void EFFECT_FAKE_STEREO::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  switch (param) {
+  case 1:
+    pd->default_value = 20.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = false;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  default: {}
+  }
+}
+
+CHAIN_OPERATOR::parameter_t EFFECT_FAKE_STEREO::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
     return(dtime_msec);
@@ -293,12 +453,13 @@ CHAIN_OPERATOR::parameter_type EFFECT_FAKE_STEREO::get_parameter(int param) cons
   return(0.0);
 }
 
-void EFFECT_FAKE_STEREO::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_FAKE_STEREO::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1:
     dtime_msec = value;
-    dtime = dtime_msec * (CHAIN_OPERATOR::parameter_type)samples_per_second() / 1000;
-    vector<deque<SAMPLE_SPECS::sample_type> >::iterator p = buffer.begin();
+    dtime = dtime_msec * (CHAIN_OPERATOR::parameter_t)samples_per_second() / 1000;
+    std::vector<std::deque<SAMPLE_SPECS::sample_t> >::iterator p = buffer.begin();
     while(p != buffer.end()) {
       if (p->size() > dtime) {
 	p->resize(static_cast<unsigned int>(dtime));
@@ -309,23 +470,24 @@ void EFFECT_FAKE_STEREO::set_parameter(int param, CHAIN_OPERATOR::parameter_type
   }
 }
 
-void EFFECT_FAKE_STEREO::init(SAMPLE_BUFFER* insample) {
+void EFFECT_FAKE_STEREO::init(SAMPLE_BUFFER* insample)
+{
   l.init(insample);
   r.init(insample);
 
-  set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
-  set_parameter(1, dtime_msec);
+  EFFECT_BASE::init(insample);
 
+  set_parameter(1, dtime_msec);
   buffer.resize(2);
 }
 
-void EFFECT_FAKE_STEREO::process(void) {
+void EFFECT_FAKE_STEREO::process(void)
+{
   l.begin(SAMPLE_SPECS::ch_left);
   r.begin(SAMPLE_SPECS::ch_right);
   while(!l.end() && !r.end()) {
-    SAMPLE_SPECS::sample_type temp_left = 0;
-    SAMPLE_SPECS::sample_type temp_right = 0;
+    SAMPLE_SPECS::sample_t temp_left = 0;
+    SAMPLE_SPECS::sample_t temp_right = 0;
     if (buffer[SAMPLE_SPECS::ch_left].size() >= dtime) {
       temp_left = buffer[SAMPLE_SPECS::ch_left].front();
       temp_right = buffer[SAMPLE_SPECS::ch_right].front();
@@ -351,15 +513,59 @@ void EFFECT_FAKE_STEREO::process(void) {
   }
 }
 
-EFFECT_REVERB::EFFECT_REVERB (CHAIN_OPERATOR::parameter_type delay_time, int surround_mode, 
-			      CHAIN_OPERATOR::parameter_type feedback_percent) 
+EFFECT_REVERB::EFFECT_REVERB (CHAIN_OPERATOR::parameter_t delay_time, int surround_mode, 
+			      CHAIN_OPERATOR::parameter_t feedback_percent) 
 {
   set_parameter(1, delay_time);
   set_parameter(2, surround_mode);
   set_parameter(3, feedback_percent);
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_REVERB::get_parameter(int param) const { 
+void EFFECT_REVERB::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  switch (param) {
+  case 1:
+    pd->default_value = 20.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = false;
+    // pd->upper_bound = 0.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 2:
+    pd->default_value = 0.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = true;
+    pd->upper_bound = 1.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = true;
+    pd->integer = true;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 3:
+    pd->default_value = 50.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = true;
+    pd->upper_bound = 100.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  default: {}
+  }
+}
+
+CHAIN_OPERATOR::parameter_t EFFECT_REVERB::get_parameter(int param) const 
+{
   switch (param) {
   case 1: 
     return(dtime_msec);
@@ -371,13 +577,14 @@ CHAIN_OPERATOR::parameter_type EFFECT_REVERB::get_parameter(int param) const {
   return(0.0);
 }
 
-void EFFECT_REVERB::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_REVERB::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     {
       dtime_msec = value;
-      dtime = dtime_msec * (CHAIN_OPERATOR::parameter_type)samples_per_second() / 1000;
-      vector<deque<SAMPLE_SPECS::sample_type> >::iterator p = buffer.begin();
+      dtime = dtime_msec * (CHAIN_OPERATOR::parameter_t)samples_per_second() / 1000;
+      std::vector<std::deque<SAMPLE_SPECS::sample_t> >::iterator p = buffer.begin();
       while(p != buffer.end()) {
 	if (p->size() > dtime) {
 	  p->resize(static_cast<unsigned int>(dtime));
@@ -397,23 +604,25 @@ void EFFECT_REVERB::set_parameter(int param, CHAIN_OPERATOR::parameter_type valu
   }
 }
 
-void EFFECT_REVERB::init(SAMPLE_BUFFER* insample) {
+void EFFECT_REVERB::init(SAMPLE_BUFFER* insample)
+{
   l.init(insample);
   r.init(insample);
 
-  set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
+  EFFECT_BASE::init(insample);
+
   set_parameter(1, dtime_msec);
 
   buffer.resize(2);
 }
 
-void EFFECT_REVERB::process(void) {
+void EFFECT_REVERB::process(void)
+{
   l.begin(SAMPLE_SPECS::ch_left);
   r.begin(SAMPLE_SPECS::ch_right);
   while(!l.end() && !r.end()) {
-    SAMPLE_SPECS::sample_type temp_left = 0.0;
-    SAMPLE_SPECS::sample_type temp_right = 0.0;
+    SAMPLE_SPECS::sample_t temp_left = 0.0;
+    SAMPLE_SPECS::sample_t temp_right = 0.0;
     if (buffer[SAMPLE_SPECS::ch_left].size() >= dtime) {
       temp_left = buffer[SAMPLE_SPECS::ch_left].front();
       temp_right = buffer[SAMPLE_SPECS::ch_right].front();
@@ -441,18 +650,73 @@ void EFFECT_REVERB::process(void) {
   }
 }
 
-EFFECT_MODULATING_DELAY::EFFECT_MODULATING_DELAY(CHAIN_OPERATOR::parameter_type delay_time, 
-						 CHAIN_OPERATOR::parameter_type feedback_percent,
+EFFECT_MODULATING_DELAY::EFFECT_MODULATING_DELAY(CHAIN_OPERATOR::parameter_t delay_time, 
 						 long int vartime_in_samples,
-						 CHAIN_OPERATOR::parameter_type lfo_freq)
+						 CHAIN_OPERATOR::parameter_t feedback_percent,
+						 CHAIN_OPERATOR::parameter_t lfo_freq)
 {
   set_parameter(1, delay_time);
-  set_parameter(2, feedback_percent);
-  set_parameter(3, vartime_in_samples);
+  set_parameter(2, vartime_in_samples);
+  set_parameter(3, feedback_percent);
   set_parameter(4, lfo_freq);
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_MODULATING_DELAY::get_parameter(int param) const { 
+void EFFECT_MODULATING_DELAY::parameter_description(int param, struct PARAM_DESCRIPTION *pd) const
+{
+  switch (param) {
+  case 1:
+    pd->default_value = 2.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = false;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 2:
+    pd->default_value = 20.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = false;
+    // pd->upper_bound = 0.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = true;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 3:
+    pd->default_value = 50.0f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = true;
+    pd->upper_bound = 100.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  case 4:
+    pd->default_value = 0.4f;
+    pd->description = get_parameter_name(param);
+    pd->bounded_above = false;
+    // pd->upper_bound = 0.0f;
+    pd->bounded_below = true;
+    pd->lower_bound = 0.0f;
+    pd->toggled = false;
+    pd->integer = false;
+    pd->logarithmic = false;
+    pd->output = false;
+    break;
+  default: {}
+  }
+}
+
+CHAIN_OPERATOR::parameter_t EFFECT_MODULATING_DELAY::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
     return(dtime_msec);
@@ -469,14 +733,18 @@ CHAIN_OPERATOR::parameter_type EFFECT_MODULATING_DELAY::get_parameter(int param)
   return(0.0);
 }
 
-void EFFECT_MODULATING_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_MODULATING_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1:
     {
       dtime_msec = value;
-      dtime = static_cast<long int>(dtime_msec * (CHAIN_OPERATOR::parameter_type)samples_per_second() / 1000);
-      assert(buffer.size() == delay_index.size());
-      assert(buffer.size() == filled.size());
+      dtime = static_cast<long int>(dtime_msec * (CHAIN_OPERATOR::parameter_t)samples_per_second() / 1000);
+      if (dtime < 1) { 
+	dtime = 1;
+      }
+      DBC_CHECK(buffer.size() == delay_index.size());
+      DBC_CHECK(buffer.size() == filled.size());
       for(int n = 0; n < static_cast<int>(buffer.size()); n++) {
 	if (dtime * 2 > static_cast<long int>(buffer[n].size())) {
 	  buffer[n].resize(dtime * 2);
@@ -501,28 +769,43 @@ void EFFECT_MODULATING_DELAY::set_parameter(int param, CHAIN_OPERATOR::parameter
   }
 }
 
-void EFFECT_MODULATING_DELAY::init(SAMPLE_BUFFER* insample) {
+void EFFECT_MODULATING_DELAY::init(SAMPLE_BUFFER* insample)
+{
   i.init(insample);
-  lfo.init(1.0 /
-	   insample->sample_rate() /
-	   insample->number_of_channels());
-  set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
+  lfo.init();
+  advance_len_rep = insample->length_in_samples();
+
   set_parameter(1, dtime_msec);
 
+  EFFECT_BASE::init(insample);
+
   filled.resize(channels(), false);
   delay_index.resize(channels(), 2 * dtime);
-  buffer.resize(channels(), vector<SAMPLE_SPECS::sample_type> (2 * dtime));
+  buffer.resize(channels(), std::vector<SAMPLE_SPECS::sample_t> (2 * dtime));
 }
 
-void EFFECT_FLANGER::process(void) {
+void EFFECT_MODULATING_DELAY::process(void)
+{
+  lfo.seek_position_in_samples_advance(advance_len_rep);
+}
+
+void EFFECT_MODULATING_DELAY::set_samples_per_second(SAMPLE_SPECS::sample_rate_t v)
+{
+  EFFECT_TIME_BASED::set_samples_per_second(v);
+  lfo.set_samples_per_second(v);
+}
+
+void EFFECT_FLANGER::process(void)
+{
+  EFFECT_MODULATING_DELAY::process();
+
   i.begin();
   while(!i.end()) {
-    SAMPLE_SPECS::sample_type temp1 = 0.0;
-    parameter_type p = vartime * lfo.value();
+    SAMPLE_SPECS::sample_t temp1 = 0.0;
+    parameter_t p = vartime * lfo.value();
     if (filled[i.channel()] == true) {
-      assert((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) >= 0);
-      assert((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) < static_cast<long int>(buffer[i.channel()].size()));
+      DBC_CHECK((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) >= 0);
+      DBC_CHECK((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) < static_cast<long int>(buffer[i.channel()].size()));
       temp1 = buffer[i.channel()][(dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2)];
     }
     *i.current() = (*i.current() * (1.0 - feedback)) + (temp1 * feedback);
@@ -537,15 +820,17 @@ void EFFECT_FLANGER::process(void) {
   }
 }
 
-void EFFECT_CHORUS::process(void) {
+void EFFECT_CHORUS::process(void)
+{
+  EFFECT_MODULATING_DELAY::process();
+
   i.begin();
-   
   while(!i.end()) {
-    SAMPLE_SPECS::sample_type temp1 = 0.0;
-    parameter_type p = vartime * lfo.value();
+    SAMPLE_SPECS::sample_t temp1 = 0.0;
+    parameter_t p = vartime * lfo.value();
     if (filled[i.channel()] == true) {
-      assert((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) >= 0);
-      assert((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) < static_cast<long int>(buffer[i.channel()].size()));
+      DBC_CHECK((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) >= 0);
+      DBC_CHECK((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) < static_cast<long int>(buffer[i.channel()].size()));
       temp1 = buffer[i.channel()][(dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2)];
     }
     buffer[i.channel()][delay_index[i.channel()]] = *i.current();
@@ -560,19 +845,21 @@ void EFFECT_CHORUS::process(void) {
   }
 }
 
-void EFFECT_PHASER::process(void) {
+void EFFECT_PHASER::process(void)
+{
+  EFFECT_MODULATING_DELAY::process();
+
   i.begin();
-   
   while(!i.end()) {
-    SAMPLE_SPECS::sample_type temp1 = 0.0;
-    parameter_type p = vartime * lfo.value();
+    SAMPLE_SPECS::sample_t temp1 = 0.0;
+    parameter_t p = vartime * lfo.value();
     if (filled[i.channel()] == true) {
-      assert((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) >= 0);
-      assert((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) < static_cast<long int>(buffer[i.channel()].size()));
+      DBC_CHECK((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) >= 0);
+      DBC_CHECK((dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2) < static_cast<long int>(buffer[i.channel()].size()));
       temp1 = buffer[i.channel()][(dtime + delay_index[i.channel()] + static_cast<long int>(p)) % (dtime * 2)];
-//          cerr << "b: "
-//    	   << (delay_index[i.channel()] + static_cast<long int>(p)) % dtime
-//    	   << "," << p << ".\n";
+      //          cerr << "b: "
+      //    	   << (delay_index[i.channel()] + static_cast<long int>(p)) % dtime
+      //    	   << "," << p << ".\n";
     }
     *i.current() = *i.current() * (1.0 - feedback) + (-1.0 * temp1 * feedback);
     buffer[i.channel()][delay_index[i.channel()]] = *i.current();
