@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audiogate.cpp: Signal gates.
-// Copyright (C) 1999-2000 Kai Vehmanen (kaiv@wakkanet.fi)
+// Copyright (C) 1999-2002 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,25 +17,32 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 // ------------------------------------------------------------------------
 
-#include <kvutils/kvu_numtostr.h>
+#include <kvu_numtostr.h>
 
+#include "samplebuffer.h"
 #include "samplebuffer_functions.h"
 #include "audiogate.h"
-#include "eca-debug.h"
+#include "eca-logger.h"
 
-void GATE_BASE::process(void) {
+GATE_BASE::~GATE_BASE(void)
+{
+}
+
+void GATE_BASE::process(void)
+{
   analyze(target);
   if (is_open() == false) {
-//      target->make_silent();
     target->length_in_samples(0);
   }
 }
 
-void GATE_BASE::init(SAMPLE_BUFFER* sbuf) { 
+void GATE_BASE::init(SAMPLE_BUFFER* sbuf)
+{ 
   target = sbuf;
 }
 
-void TIME_CROP_GATE::analyze(SAMPLE_BUFFER* sbuf) {
+void TIME_CROP_GATE::analyze(SAMPLE_BUFFER* sbuf)
+{
   if (curtime >= btime) {
     if (btime == etime) open_gate();
     else if (curtime < etime) open_gate();
@@ -44,20 +51,21 @@ void TIME_CROP_GATE::analyze(SAMPLE_BUFFER* sbuf) {
   else 
     close_gate();
 
-  curtime += sbuf->length_in_seconds();
+  curtime += static_cast<double>(sbuf->length_in_samples() / samples_per_second());
 }
 
-TIME_CROP_GATE::TIME_CROP_GATE (CHAIN_OPERATOR::parameter_type open_at, CHAIN_OPERATOR::parameter_type duration) {
+TIME_CROP_GATE::TIME_CROP_GATE (CHAIN_OPERATOR::parameter_t open_at, CHAIN_OPERATOR::parameter_t duration) {
   btime = open_at;
   etime = btime + duration;
   curtime = 0.0;
   
-  ecadebug->msg("(audiogate) Time crop gate created; opens at " +
-		kvu_numtostr(btime) + " seconds and stays open for " +
-		kvu_numtostr(duration) + " seconds.\n");
+  ECA_LOG_MSG(ECA_LOGGER::info, "(audiogate) Time crop gate created; opens at " +
+	      kvu_numtostr(btime) + " seconds and stays open for " +
+	      kvu_numtostr(duration) + " seconds.\n");
 }
 
-CHAIN_OPERATOR::parameter_type TIME_CROP_GATE::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t TIME_CROP_GATE::get_parameter(int param) const 
+{ 
   switch (param) {
   case 1: 
     return(btime);
@@ -67,7 +75,8 @@ CHAIN_OPERATOR::parameter_type TIME_CROP_GATE::get_parameter(int param) const {
   return(0.0);
 }
 
-void TIME_CROP_GATE::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void TIME_CROP_GATE::set_parameter(int param, CHAIN_OPERATOR::parameter_t value) 
+{
   switch (param) {
   case 1: 
     btime = value;
@@ -79,9 +88,10 @@ void TIME_CROP_GATE::set_parameter(int param, CHAIN_OPERATOR::parameter_type val
   }
 }
 
-THRESHOLD_GATE::THRESHOLD_GATE (CHAIN_OPERATOR::parameter_type threshold_openlevel, 
-				CHAIN_OPERATOR::parameter_type threshold_closelevel,
-				bool use_rms) {
+THRESHOLD_GATE::THRESHOLD_GATE (CHAIN_OPERATOR::parameter_t threshold_openlevel, 
+				CHAIN_OPERATOR::parameter_t threshold_closelevel,
+				bool use_rms) 
+{
   openlevel = threshold_openlevel / 100.0;
   closelevel = threshold_closelevel / 100.0;
   rms = use_rms;
@@ -89,18 +99,19 @@ THRESHOLD_GATE::THRESHOLD_GATE (CHAIN_OPERATOR::parameter_type threshold_openlev
   is_opened = is_closed = false;
 
   if (rms) {
-    ecadebug->msg("(audiogate) Threshold gate created; open threshold " +
-		  kvu_numtostr(openlevel * 100) + "%, close threshold " +
-		  kvu_numtostr(closelevel * 100) + "%, using RMS volume.");
+    ECA_LOG_MSG(ECA_LOGGER::info, "(audiogate) Threshold gate created; open threshold " +
+		kvu_numtostr(openlevel * 100) + "%, close threshold " +
+		kvu_numtostr(closelevel * 100) + "%, using RMS volume.");
   }
   else {
-    ecadebug->msg("(audiogate) Threshold gate created; open threshold " +
-		  kvu_numtostr(openlevel * 100) + "%, close threshold " +
-		  kvu_numtostr(closelevel * 100) + "%, using peak volume.");
+    ECA_LOG_MSG(ECA_LOGGER::info, "(audiogate) Threshold gate created; open threshold " +
+		kvu_numtostr(openlevel * 100) + "%, close threshold " +
+		kvu_numtostr(closelevel * 100) + "%, using peak volume.");
   }
 }
 
-void THRESHOLD_GATE::analyze(SAMPLE_BUFFER* sbuf) {
+void THRESHOLD_GATE::analyze(SAMPLE_BUFFER* sbuf)
+{
   if (rms == true) avolume = SAMPLE_BUFFER_FUNCTIONS::RMS_volume(*sbuf) /
 		     SAMPLE_SPECS::max_amplitude;
   else avolume = SAMPLE_BUFFER_FUNCTIONS::average_amplitude(*sbuf) / SAMPLE_SPECS::max_amplitude;
@@ -108,20 +119,21 @@ void THRESHOLD_GATE::analyze(SAMPLE_BUFFER* sbuf) {
   if (is_opened == false) {
     if (avolume > openlevel) { 
       open_gate();
-      ecadebug->msg(ECA_DEBUG::user_objects, "(audiogate) Threshold gate opened.");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(audiogate) Threshold gate opened.");
       is_opened = true;
     }
   }
   else if (is_closed == false) {
     if (avolume < closelevel) { 
       close_gate();
-      ecadebug->msg(ECA_DEBUG::user_objects, "(audiogate) Threshold gate closed.");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(audiogate) Threshold gate closed.");
       is_closed = true;
     }
   }
 }
 
-CHAIN_OPERATOR::parameter_type THRESHOLD_GATE::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t THRESHOLD_GATE::get_parameter(int param) const
+{ 
   switch (param) {
   case 1: 
     return(openlevel * 100.0);
@@ -134,7 +146,9 @@ CHAIN_OPERATOR::parameter_type THRESHOLD_GATE::get_parameter(int param) const {
   return(0.0);
 }
 
-void THRESHOLD_GATE::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void THRESHOLD_GATE::set_parameter(int param, CHAIN_OPERATOR::parameter_t value) 
+{
+
   switch (param) {
   case 1: 
     openlevel = value / 100.0;
@@ -148,6 +162,3 @@ void THRESHOLD_GATE::set_parameter(int param, CHAIN_OPERATOR::parameter_type val
     break;
   }
 }
-
-
-
