@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audioio-cdr.cpp: CDDA/CDR audio file format input/output
-// Copyright (C) 1999 Kai Vehmanen (kaiv@wakkanet.fi)
+// Copyright (C) 1999,2001 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,50 +17,53 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 // ------------------------------------------------------------------------
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <cmath>
 #include <cstdio>
 #include <string>
 #include <cstring>
 #include <cassert>
-#include <sys/stat.h>
-#include <unistd.h>
+#include <climits> /* LONG_MAX */
+#include <sys/stat.h> /* stat() */
+#include <unistd.h> /* stat() */
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h> /* off_t */
+#else
+typedef long int off_t;
+#endif
 
-#include "samplebuffer.h"
-#include "audioio-types.h"
+#include <kvu_dbc.h>
+
+#include "sample-specs.h"
 #include "audioio-cdr.h"
-#include "eca-debug.h"
+#include "eca-logger.h"
 
-CDRFILE::CDRFILE(const string& name) {
-  label(name);
+CDRFILE::CDRFILE(const std::string& name)
+{
+  set_label(name);
 }
 
-CDRFILE::~CDRFILE(void) {
-  if (is_open() == true)
+CDRFILE::~CDRFILE(void)
+{
+  if (is_open() == true) {
     close();
+  }
 }
 
-void CDRFILE::format_query(void) {
-  // --------
-  // require:
-  assert(!is_open());
-  // --------
-
-  struct stat temp;
-  stat(label().c_str(), &temp);
-  length_in_samples(temp.st_size / frame_size());
-
-  // -------
-  // ensure:
-  assert(!is_open());
-  // -------
+CDRFILE* CDRFILE::clone(void) const
+{
+  CDRFILE* target = new CDRFILE();
+  for(int n = 0; n < number_of_params(); n++) {
+    target->set_parameter(n + 1, get_parameter(n + 1));
+  }
+  return(target);
 }
 
-void CDRFILE::open(void) throw(AUDIO_IO::SETUP_ERROR &) { 
-  // --------
-  // require:
-  assert(!is_open());
-  // --------
-
+void CDRFILE::open(void) throw(AUDIO_IO::SETUP_ERROR &)
+{
   set_channels(2);
   set_sample_format(ECA_AUDIO_FORMAT::sfmt_s16_be);
   set_samples_per_second(44100);
@@ -68,7 +71,7 @@ void CDRFILE::open(void) throw(AUDIO_IO::SETUP_ERROR &) {
   switch(io_mode()) {
   case io_read:
     {
-      fobject = ::fopen(label().c_str(),"rb");
+      fobject = std::fopen(label().c_str(),"rb");
       if (!fobject)
 	throw(SETUP_ERROR(SETUP_ERROR::io_mode, "AUDIOIO-CDR: Can't open " + label() + " for reading."));
       set_length_in_bytes();
@@ -76,16 +79,16 @@ void CDRFILE::open(void) throw(AUDIO_IO::SETUP_ERROR &) {
     }
   case io_write: 
     {
-      fobject = ::fopen(label().c_str(),"wb");
+      fobject = std::fopen(label().c_str(),"wb");
       if (!fobject) 
 	throw(SETUP_ERROR(SETUP_ERROR::io_mode, "AUDIOIO-CDR: Can't open " + label() + " for writing."));
       break;
     }
   case io_readwrite:
     {
-      fobject = ::fopen(label().c_str(),"r+b");
+      fobject = std::fopen(label().c_str(),"r+b");
       if (!fobject) {
-	fobject = ::fopen(label().c_str(),"w+b");
+	fobject = std::fopen(label().c_str(),"w+b");
 	if (!fobject)
 	  throw(SETUP_ERROR(SETUP_ERROR::io_mode, "AUDIOIO-CDR: Can't open " + label() + " for read&write."));
       }
@@ -93,37 +96,68 @@ void CDRFILE::open(void) throw(AUDIO_IO::SETUP_ERROR &) {
       break;
     }
   }
-  toggle_open_state(true);
-  seek_position();
+
+  AUDIO_IO::open();
 }
 
-void CDRFILE::close(void) { 
+void CDRFILE::close(void)
+{ 
   if (io_mode() != io_read)
     pad_to_sectorsize();
 
-  ::fclose(fobject);
-  toggle_open_state(false);
+  std::fclose(fobject);
+
+  AUDIO_IO::close();
 }
 
 bool CDRFILE::finished(void) const {
- if (ferror(fobject) ||
-     feof(fobject))
+ if (std::ferror(fobject) ||
+     std::feof(fobject))
    return true;
 
  return false;
 }
 
-long int CDRFILE::read_samples(void* target_buffer, long int samples) {
-  return(::fread(target_buffer, frame_size(), samples, fobject));
+long int CDRFILE::read_samples(void* target_buffer, long int samples)
+{
+  return(std::fread(target_buffer, frame_size(), samples, fobject));
 }
 
 void CDRFILE::write_samples(void* target_buffer, long int samples) {
-  ::fwrite(target_buffer, frame_size(), samples, fobject);
+  std::fwrite(target_buffer, frame_size(), samples, fobject);
 }
 
 void CDRFILE::seek_position(void) {
-  if (is_open())
-    ::fseek(fobject, position_in_samples() * frame_size(), SEEK_SET);
+  if (is_open() == true) {
+    off_t curpos_rep = position_in_samples() * frame_size();
+    DBC_CHECK(curpos_rep >= 0);
+/* fseeko doesn't seem to work with glibc 2.1.x */
+#if _LARGEFILE_SOURCE
+    off_t seekpos = 0;
+    off_t seekstep = 0;
+    int whence = SEEK_SET;
+    while(curpos_rep - seekpos > 0) {
+      if (curpos_rep - seekpos > LONG_MAX)
+	seekstep = LONG_MAX;
+      else
+	seekstep = curpos_rep - seekpos;
+
+      // std::cerr << "(audioio-cdr) fw-seeking from " << seekpos << " to " << seekpos+seekstep << std::endl;
+      int res = std::fseek(fobject, seekstep, whence);
+      if (res != 0) {
+	  ECA_LOG_MSG(ECA_LOGGER::info, "(audioio-cdr) fseek() error! (lfs).");
+	  curpos_rep = 0;
+	  std::fseek(fobject, 0, SEEK_SET);
+	  break;
+      }
+      if (seekpos == 0) whence = SEEK_CUR;
+      seekpos += seekstep;
+    }
+#else
+    DBC_CHECK(sizeof(long int) == sizeof(off_t));
+    std::fseek(fobject, curpos_rep, SEEK_SET);
+#endif
+  }
 }
 
 void CDRFILE::pad_to_sectorsize(void) {
@@ -134,12 +168,14 @@ void CDRFILE::pad_to_sectorsize(void) {
     return;
   }
   for(int n = 0; n < padsamps; n++) ::fputc(0,fobject);
-  assert(::ftell(fobject) %  CDRFILE::sectorsize == 0);
+
+  DBC_DECLARE(long int endpos);
+  DBC_DECLARE(endpos = std::ftell(fobject));
+  DBC_CHECK((endpos %  CDRFILE::sectorsize) == 0);
 }
 
 void CDRFILE::set_length_in_bytes(void) {
-  long int save = ::ftell(fobject);
-  ::fseek(fobject,0,SEEK_END);
-  length_in_samples(ftell(fobject) / frame_size());
-  ::fseek(fobject,save,SEEK_SET);
+  struct stat temp;
+  stat(label().c_str(), &temp);
+  set_length_in_samples(temp.st_size / frame_size());
 }
