@@ -18,36 +18,62 @@
 // ------------------------------------------------------------------------
 
 #include <string>
-#include <unistd.h>
+#include <unistd.h> /* stat() */
+#include <sys/stat.h> /* stat() */
 
-#include <kvutils/kvu_numtostr.h>
+#include <kvu_numtostr.h>
 
 #include "audioio-mikmod.h"
-#include "eca-debug.h"
+#include "eca-logger.h"
 
 string MIKMOD_INTERFACE::default_mikmod_cmd = "mikmod -d stdout -o 16s -q -f %s -p 0 --noloops %f";
 
-void MIKMOD_INTERFACE::set_mikmod_cmd(const string& value) { MIKMOD_INTERFACE::default_mikmod_cmd = value; }
+void MIKMOD_INTERFACE::set_mikmod_cmd(const std::string& value) { MIKMOD_INTERFACE::default_mikmod_cmd = value; }
 
-MIKMOD_INTERFACE::MIKMOD_INTERFACE(const string& name) {
+MIKMOD_INTERFACE::MIKMOD_INTERFACE(const std::string& name) {
   finished_rep = false;
-  set_sample_format(ECA_AUDIO_FORMAT::sfmt_s16_le);
 }
 
-MIKMOD_INTERFACE::~MIKMOD_INTERFACE(void) { close(); }
+MIKMOD_INTERFACE::~MIKMOD_INTERFACE(void) 
+{
+  if (is_open() == true) {
+    close();
+  }
+}
 
-void MIKMOD_INTERFACE::open(void) throw (AUDIO_IO::SETUP_ERROR &) { 
+void MIKMOD_INTERFACE::open(void) throw (AUDIO_IO::SETUP_ERROR &)
+{
+  std::string urlprefix;
   triggered_rep = false;
-  toggle_open_state(true); 
+
+  struct stat buf;
+  int ret = ::stat(label().c_str(), &buf);
+  if (ret != 0) {
+    size_t offset = label().find_first_of("://");
+    if (offset == std::string::npos) {
+      throw(SETUP_ERROR(SETUP_ERROR::io_mode, "AUDIOIO-MIKMOD: Can't open file " + label() + "."));
+    }
+    else {
+      urlprefix = std::string(label(), 0, offset);
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(audioio-mikmod) Found url; protocol '" + urlprefix + "'.");
+    }
+  }
+
+  /* s16 samples, 2 channels, srate configurable */
+  set_sample_format(ECA_AUDIO_FORMAT::sfmt_s16_le);
+  set_channels(2);
+
+  AUDIO_IO::open();
 }
 
-void MIKMOD_INTERFACE::close(void) {
+void MIKMOD_INTERFACE::close(void)
+{
   if (pid_of_child() > 0) {
     if (io_mode() == io_read) {
       kill_mikmod();
     }
   }
-  toggle_open_state(false);
+  AUDIO_IO::close();
 }
 
 long int MIKMOD_INTERFACE::read_samples(void* target_buffer, long int samples) {
@@ -59,23 +85,26 @@ long int MIKMOD_INTERFACE::read_samples(void* target_buffer, long int samples) {
   bytes_read_rep = ::fread(target_buffer, 1, frame_size() * samples, f1_rep);
   if (bytes_read_rep < samples * frame_size() || bytes_read_rep == 0) {
     if (position_in_samples() == 0) 
-      ecadebug->msg(ECA_DEBUG::info, "(audioio-mikmod) Can't start process \"" + MIKMOD_INTERFACE::default_mikmod_cmd + "\". Please check your ~/.ecasoundrc.");
+      ECA_LOG_MSG(ECA_LOGGER::info, "(audioio-mikmod) Can't start process \"" + MIKMOD_INTERFACE::default_mikmod_cmd + "\". Please check your ~/.ecasoundrc.");
     finished_rep = true;
+    triggered_rep = false;
   }
   else finished_rep = false;
   return(bytes_read_rep / frame_size());
 }
 
-void MIKMOD_INTERFACE::seek_position(void) {
+void MIKMOD_INTERFACE::seek_position(void)
+{
   if (triggered_rep == true) {
     if (io_mode() == io_read) {
       kill_mikmod();
     }
   }
+  set_position_in_samples(0);
 }
 
 void MIKMOD_INTERFACE::kill_mikmod(void) {
-  ecadebug->msg(ECA_DEBUG::user_objects, "(audioio-mikmod) Cleaning mikmod-child with pid " + kvu_numtostr(pid_of_child()) + ".");
+  ECA_LOG_MSG(ECA_LOGGER::user_objects, "(audioio-mikmod) Cleaning mikmod-child with pid " + kvu_numtostr(pid_of_child()) + ".");
   clean_child();
   triggered_rep = false;
 }
@@ -87,7 +116,10 @@ void MIKMOD_INTERFACE::fork_mikmod(void) {
   fork_child_for_read();
   if (child_fork_succeeded() == true) {
     fd_rep = file_descriptor();
-    f1_rep = fdopen(fd_rep, "r");
-    if (f1_rep == 0) finished_rep = true;
+    f1_rep = fdopen(fd_rep, "r"); /* not part of <cstdio> */
+    if (f1_rep == 0) {
+      finished_rep = true;
+      triggered_rep = false;
+    }
   }
 }
