@@ -1,5 +1,5 @@
-#ifndef _OSCILLATOR_H
-#define _OSCILLATOR_H
+#ifndef INCLUDED_OSCILLATOR_H
+#define INCLUDED_OSCILLATOR_H
 
 #include <cmath>
 #include <string>
@@ -16,8 +16,8 @@ class OSCILLATOR : public CONTROLLER_SOURCE {
 
  public:
 
-  virtual string name(void) const = 0;
-  virtual parameter_type value(void) = 0;
+  virtual std::string name(void) const = 0;
+  virtual parameter_t value(void) = 0;
 
   /**
    * Constructor
@@ -25,22 +25,22 @@ class OSCILLATOR : public CONTROLLER_SOURCE {
    * @param freq Oscillator frequency
    * @param phase Initial phase, multiple of pi
    */
-  OSCILLATOR(parameter_type freq = 0, parameter_type initial_phase = 0) { 
+  OSCILLATOR(parameter_t freq = 0, parameter_t initial_phase = 0) { 
     freq_value = freq;
     phase_value = initial_phase * M_PI;
   }
 
  private:
   
-  parameter_type freq_value, phase_value;
+  parameter_t freq_value, phase_value;
 
  protected:
 
-  parameter_type phase_offset(void) const { return(phase_value); }  
-  parameter_type frequency(void) const { return(freq_value); }
+  parameter_t phase_offset(void) const { return(phase_value); }  
+  parameter_t frequency(void) const { return(freq_value); }
 
-  void phase_offset(parameter_type v) { phase_value = v; }
-  void frequency(parameter_type v) { freq_value = v; }
+  void phase_offset(parameter_t v) { phase_value = v; }
+  void frequency(parameter_t v) { freq_value = v; }
 };
 
 #endif
